/*
 * cmrfinp.c
 *
 * Device server specific code for RF ramping control boards.
 *
 *	12.03.2003 - GG: first implementation
 */

/*
 * system includes
 */
#include <setsys.h>
#include <stdio.h>
#include <math.h>    /* for fabs() */
#include <module.h>
#include <modes.h>
#include <stdlib.h>  /* for atoi() */
#include <string.h>  /* for strcmp() */
#include <errno.h>

/*
 * elettra includes
 */
#include <cherrs.h>   /* error numbers */
#include <cherrt.h>   /* error types */
#include "chlogs.h"   /* logging funcs */
#include "chmodule.h"

/*
 * felix includes
 */
#include "chdevsrv.h"  /* generic device server stuff */
#include "chgetres.h"  /* resource reading stuff */

/*
 * driver includes
 */
#include "chadio.h"  /* ioctl codes */

/*
 * device server includes
 */
#include "chrfinp.h"  /* specific device server stuff */


/*
 * global vars
 */
resource_type resources;
short start_vlt, end_vlt;
int start_time, end_time;    /* loop starting and ending time in ticks */
int enable_flag = 0;
int start_time_fast,end_time_fast;
short end_vlt_fast = 0;
int delta;

/*
 * vars from generic part
 */
extern char loc_msg[200];
extern srvmod_type *srv_mod;  /* pointer to server module data structure */
extern loop_type *loop_var;
extern call_type *call_pnt;
extern str_typ res_err;       /* resource reading global error string */

/*
 * function prototypes
 */
void iniz_hw(void);       /* HW initializations */
void iniz_sw(void);       /* SW initializations */
int check_enable(void);		/* check if enabled */
void felix_terminat(int); /* specific part resources deallocation */
void go_on_loop(int);     /* server loops stuff */
void go_on_loop0(void);		/* voltage ramping loop */
void go_on_loop0(void);		/* fast ramping loop */
int read_resources(char *);  /* routine di lettura delle risorse */

/*
 * server routines
 */
void abort_wp(void);	/* abort ramping */
void vlt_rr(void);		/* read cavity voltage */
void vltset_rr(void);	/* read cavity voltage setting */
void linrmp_wrrr(void);	/* start voltage ramp */
void fstrmp_wp(void);		/* start fast ramp */
void dvlt_wr(void);			/* set cavity voltage (DANGEROUS) */
void enable_wb(void);		/* enable power */
void stat_rf(void);			/* read status (enable/ramping) */
void limits_rr(void);		/* read limits */

/* 
 * action mode table 
 */
act_mod_typ amtab[] = {
  "ABORT",  "WP",     abort_wp,    /* 1 */
  "VLT",    "RR",     vlt_rr,      /* 2 */
  "VLTSET", "RR",     vltset_rr,   /* 3 */
  "LINRMP", "WRRR",   linrmp_wrrr, /* 4 */
  "FSTRMP", "WP",     fstrmp_wp,	 /* 5 */	
  "DVLT",   "WR",     dvlt_wr,     /* 6 */
  "ENABLE", "WB",     enable_wb,   /* 7 */
	"STAT",   "RF",     stat_rf,     /* 8 */	
  "LIMITS", "RR",     limits_rr    /* 9 */
};

/*
 * felix_terminat()
 * specific part resources deallocation
 */
void felix_terminat(int sig)
{
  return;
}


/*
 * read_resources()
 * open and read the resources file
 */
int read_resources(char *filename)
{
  int i;

  if (read_res(filename) < 0) {
    printf("%s: cannot read resource file %s\n", \
           gettime(), filename);
    return -1;  /* some file handling error? */
  }
  
  if (get_res(resources.adio.devname, "adio.devname", STRING) || \
      get_res(&resources.adio.in_mask, "adio.in_mask", USHORT) || \
      get_res(&resources.adio.enable_mask,"adio.enable_mask", USHORT) || \
			get_res(&resources.adio.dac.a, "adio.dac.a", DOUBLE) || \
			get_res(&resources.adio.dac.b, "adio.dac.b", DOUBLE) || \
      get_res(&resources.adio.dac.c, "adio.dac.c", DOUBLE) || \
      get_res(&resources.adio.dac.d, "adio.dac.d", DOUBLE) || \
      get_res(&resources.adio.dac.min, "adio.dac.min", DOUBLE) || \
			get_res(&resources.adio.dac.max, "adio.dac.max", DOUBLE) || \
			get_res(&resources.adio.dac_set.a, "adio.dac_set.a", DOUBLE) || \
			get_res(&resources.adio.dac_set.b, "adio.dac_set.b", DOUBLE) || \
      get_res(&resources.adio.dac_set.c, "adio.dac_set.c", DOUBLE) || \
      get_res(&resources.adio.dac_set.d, "adio.dac_set.d", DOUBLE) || \
      get_res(&resources.adio.dac_set.min, "adio.dac_set.min", DOUBLE) || \
			get_res(&resources.adio.dac_set.max, "adio.dac_set.max", DOUBLE) || \
			get_res(&resources.adio.adc.a, "adio.adc.a", DOUBLE) || \
      get_res(&resources.adio.adc.b, "adio.adc.b", DOUBLE) || \
      get_res(&resources.adio.adc.c, "adio.adc.c", DOUBLE) || \
      get_res(&resources.adio.adc.d, "adio.adc.d", DOUBLE) || \
      get_res(&resources.adio.adc.min, "adio.adc.min", DOUBLE) || \
			get_res(&resources.adio.adc.max, "adio.adc.max", DOUBLE) || \
      get_res(&resources.adio.max_vel, "adio.max_vel", DOUBLE) || \
			get_res(&resources.adio.max_fast_ramp, "adio.max_fast_ramp", DOUBLE) || \
			get_res(&resources.adio.fast_time, "adio.fast_time", INTEGER)) {
    return -1;
  }
  
#ifdef DEBUGRES
  fprintf(stderr, "adio.devname = %s\n", resources.adio.devname);
  fprintf(stderr, "adio.in_mask = 0x%x\n", resources.adio.in_mask);
  fprintf(stderr, "adio.enable_mask = 0x%x\n", resources.adio.enable_mask);
  fprintf(stderr, "adio.dac.a = %f\n", resources.adio.dac.a);
  fprintf(stderr, "adio.dac.b = %f\n", resources.adio.dac.b);
  fprintf(stderr, "adio.dac.c = %f\n", resources.adio.dac.c);
  fprintf(stderr, "adio.dac.d = %f\n", resources.adio.dac.d);
  fprintf(stderr, "adio.dac.min = %f\n", resources.adio.dac.min);	
  fprintf(stderr, "adio.dac.max = %f\n", resources.adio.dac.max);
  fprintf(stderr, "adio.dac_set.a = %f\n", resources.adio.dac_set.a);
  fprintf(stderr, "adio.dac_set.b = %f\n", resources.adio.dac_set.b);
  fprintf(stderr, "adio.dac_set.c = %f\n", resources.adio.dac_set.c);
  fprintf(stderr, "adio.dac_set.d = %f\n", resources.adio.dac_set.d);
  fprintf(stderr, "adio.dac_set.min = %f\n", resources.adio.dac_set.min);	
  fprintf(stderr, "adio.dac_set.max = %f\n", resources.adio.dac_set.max);
  fprintf(stderr, "adio.adc.a = %f\n", resources.adio.adc.a);
  fprintf(stderr, "adio.adc.b = %f\n", resources.adio.adc.b);
  fprintf(stderr, "adio.adc.c = %f\n", resources.adio.adc.c);
  fprintf(stderr, "adio.adc.d = %f\n", resources.adio.adc.d);
  fprintf(stderr, "adio.adc.min = %f\n", resources.adio.adc.min);	
  fprintf(stderr, "adio.adc.max = %f\n", resources.adio.adc.max);			
  fprintf(stderr, "adio.max_vel = %f\n", resources.adio.max_vel);
  fprintf(stderr, "adio.max_fast_ramp = %f\n", resources.adio.max_fast_ramp);
  fprintf(stderr, "adio.fast_time = %d\n", resources.adio.fast_time);		
	
#endif
  free_res();  /* free memory */

  return 0;

}  /* end read_resources() */


/*
 * iniz_hw()
 * hardware inizialization
 */
void iniz_hw(void)
{
  /* define the digital inputs */
  if (equip(resources.adio.devname, ADIO_DIGIT, \
									(char *)&resources.adio.in_mask)) {
    printf("%s: error during initialization: dev= %s, errno= %d\n", \
      	   gettime(), resources.adio.devname, errno);
    die(errno);
  }

}  /* end iniz_hw() */


/*
 * iniz_sw()
 * Software inizialization
 */
void iniz_sw(void)
{
	int int_val;
	double double_vlt;
	
	/* calculate the last value to set on the DAC during fast ramp */
	double_vlt = resources.adio.max_fast_ramp;
  double_vlt = double_vlt * double_vlt * double_vlt * resources.adio.dac.a + \
							 double_vlt * double_vlt * resources.adio.dac.b + \
               double_vlt * resources.adio.dac.c + \
							 resources.adio.dac.d;
  /* rounding */
  if (double_vlt >= 0)
    int_val = (int)(double_vlt + 0.5);
  else
    int_val = (int)(double_vlt - 0.5);

  /* if values overcome the hard limits set the limits */
  if (int_val > ADIO_HIGHTHR)
    end_vlt_fast = ADIO_HIGHTHR;
  else if (int_val < -ADIO_LOWTHR)
    end_vlt_fast = -ADIO_LOWTHR;
  else
   end_vlt_fast = (short)int_val;

}	/* end iniz_sw() */


/*
 * check_enable()
 * Check if enabled (to change voltage)
 */
int check_enable(void)
{
	short s_data;
	
	/* read the status from ADIO */
  if (equip(resources.adio.devname, ADIO_DIGIN , (char *)&s_data) != 0) {
    set_err(call_pnt, errno, ETERROR + ETAPPL, \
            "Open/Ioctl Error in Devsrv:", resources.adio.devname, ADIO_DIGIN);
		return -1;
	}

	if ((~s_data) & resources.adio.in_mask)
  	return 0;
	else { 
		set_err(call_pnt, ER_BADSTATUS, ETERROR + ETAPPL, \
            "RFinp disabled", "", 0);
		return -1;
	}

}	/* end check_enable()


/*
 * abort_wp()
 * Abort ramping
 */
void abort_wp(void)
{
  int stop_flag = 0;
	
	/* abort any active loop */
  if (loop_var[VLT_RAMP].loop) {
    stop_loop(VLT_RAMP);
		stop_flag++;
	}
	if (loop_var[FAST_RAMP].loop) {
    stop_loop(FAST_RAMP);		
  	stop_flag++;
	}
	
	if (!stop_flag) {
    set_err(call_pnt, ER_RAMPABORT, ETERROR+ETAPPL, \
      	    "Ramp Not Running ", "", 0 );
    return;
  }
  call_pnt->bytecount = 0;
	
}  /* end abort_wp() */


/*
 * vlt_rr()
 * Read cavity voltage
 */
void vlt_rr(void)
{
  short s_data;
  double d_data;

  /* read the ADC */
  if (equip(resources.adio.devname, ADIO_RDADC , (char *)&s_data) != 0) {
    set_err(call_pnt, errno, ETERROR + ETAPPL, \
      	    "Open/Ioctl Error in Devsrv:", resources.adio.devname, ADIO_RDADC);
    return;
  }

  /* calculate voltage value with calibration parameters */
  d_data = (double) s_data;
  d_data = d_data * d_data * d_data * resources.adio.adc.a + \
					 d_data * d_data * resources.adio.adc.b + \
					 d_data * resources.adio.adc.c + \
					 resources.adio.adc.d;

  /* return value */
  call_pnt->bytecount = sizeof(d_data);
	*((double*)call_pnt->data) = d_data;

#ifdef DEBUG
  fprintf(stderr, "vlt_rr(): data = 0x%x - %3.3f, bytecount = %d\n", \
          s_data, *(double *)call_pnt->data, call_pnt->bytecount);
#endif
	
}	/* end vlt_rr() */


/*
 * vltset_rr()
 * Read cavity setting voltage
 */
void vltset_rr(void)
{
  short s_data;
  double d_data;

  /* read the DAC */
  if (equip(resources.adio.devname, ADIO_RDDAC, (char *)&s_data) != 0) {
    set_err(call_pnt, errno, ETERROR+ETAPPL, \
      	    "Open/Ioctl Error in Devsrv:", resources.adio.devname, ADIO_RDDAC);
    return;
  }

  /* calculate voltage value setting */
  d_data = s_data;
  d_data = d_data * d_data * d_data * resources.adio.dac_set.a + \
					 d_data * d_data * resources.adio.dac_set.b + \
					 d_data * resources.adio.dac_set.c + \
					 resources.adio.dac_set.d;

  /* return value */
  call_pnt->bytecount = sizeof(d_data);
  *((double*)call_pnt->data) = d_data;

#ifdef DEBUG
  fprintf(stderr, "vltset_rr(): data = 0x%x - %3.3f, bytecount = %d\n", \
          s_data, *(double *)call_pnt->data, call_pnt->bytecount);
#endif
	
}  /* end vltset_rr() */

    
/*
 * dvlt_wr()
 * Direct voltage setting (DANGEROUS)
 */
void dvlt_wr(void)
{
  short s_data;
  double d_data;
  int int_val;
  
	if (check_enable() < 0)
		return;
	
	/* ramp already running ? */
  if (loop_var[VLT_RAMP].loop) {
    set_err(call_pnt, ER_POINTBUSY, ETERROR+ETAPPL, \
      	    "Ramp Already Running ", "", 0 );
    return;
  }
	/* ramp already running ? */
  if (loop_var[FAST_RAMP].loop) {
    set_err(call_pnt, ER_POINTBUSY, ETERROR+ETAPPL, \
      	    "Ramp Already Running ", "", 0 );
    return;
  }	
	
  /* get the value from the data module */
  d_data = *((double *)call_pnt->data);

  /* max min check */
  if ((d_data > resources.adio.dac.max) || \
			(d_data < resources.adio.dac.min)) {
    set_err(call_pnt, ER_OUTOFRANGE, ETERROR + ETAPPL, \
            "Value Out Of Range: ", "", (int)d_data);
    return;
  }

  /* set DAC output value */
  d_data = d_data * d_data * d_data * resources.adio.dac.a + \
					 d_data * d_data * resources.adio.dac.b + \
					 d_data * resources.adio.dac.c + \
					 resources.adio.dac.d;
      
  /* rounding */
  if (d_data >= 0)
    int_val = (int)(d_data + 0.5);
  else
    int_val = (int)(d_data - 0.5);
  
  /* if values overcome the limits set the limits */
  if ( (int_val > ADIO_HIGHTHR) )
    s_data = ADIO_HIGHTHR;
  else if ( (int_val < -ADIO_LOWTHR) )
    s_data = -ADIO_LOWTHR;
  else
    s_data = (short)int_val;
  
  /* set the DAC */
  if (equip(resources.adio.devname, ADIO_WRDAC, (char *)&s_data) != 0) {
    set_err(call_pnt, errno, ETERROR+ETAPPL, \
      	    "Open/Ioctl Error in Devsrv:", resources.adio.devname, ADIO_WRDAC);
    return;
  }
  call_pnt->bytecount = 0;

#ifdef DEBUG
  fprintf(stderr, "dvlt_wr(): data = 0x%x - %3.3f, bytecount = %d\n", \
          s_data,*(double *)call_pnt->data, call_pnt->bytecount);
#endif
	
}  /* end dvlt_wr() */


/*
 * limits_rr()
 */
void limits_rr(void)
{
  double *d_data;

  d_data = (double *)call_pnt->data;  /* point data */
  *(d_data++) = resources.adio.dac.min;  /* min voltage (kV) */
  *(d_data++) = resources.adio.dac.max;  /* max voltage (kV) */
  *d_data = resources.adio.max_vel;      /* max rate (kV*sec) */	

  call_pnt->bytecount = 3 * sizeof(*d_data);  /* set bytecount */
}  /* end limits_rr() */


/*
 * linrmp_wrrr()
 * Start ramping
 */
void linrmp_wrrr(void)
{
  short short_val;
  double double_vlt, double_vel, double1;
  int int_val;
  double time_to_finish;

	if (check_enable() < 0)
		return;

  /* fast ramp already running ? */
  if (loop_var[FAST_RAMP].loop)  {
    set_err(call_pnt, ER_POINTBUSY, ETERROR+ETAPPL, \
      	    "Ramp Already Running ", "", 0 );
    return;
  }
	
	if (loop_var[VLT_RAMP].loop)
		stop_loop(VLT_RAMP); 

  /* read the voltage value and velocity from the data module */
  double_vlt = *((double *)call_pnt->data);
  double_vel = *((double *)&call_pnt->data[sizeof(double_vlt)]);
  double1 = double_vlt;

#ifdef MYDEBUG
  fprintf(stderr, "linrmp_wr(): vlt = %f [kV], speed = %f [kV/s]\n", \
          double_vlt, double_vel);
#endif
  
  /* limits check */
  if ((double_vlt > resources.adio.dac.max) || \
			(double_vlt < resources.adio.dac.min)) {
    set_err(call_pnt, ER_OUTOFRANGE, ETERROR + ETAPPL, \
            "Value Out Of Range: ", "", (int)double_vlt);
    return;
  }

  if ((double_vel > resources.adio.max_vel) || (double_vel <= 0.)) {
    set_err(call_pnt, ER_OUTOFRANGE, ETERROR + ETAPPL, \
            "Slope Out Of Range: ", "", (int)double_vel );
    return;
  }

  /* calculate the value to set on the DAC, calibrated */
  double_vlt = double_vlt * double_vlt * double_vlt * resources.adio.dac.a + \
							 double_vlt * double_vlt * resources.adio.dac.b + \
               double_vlt * resources.adio.dac.c + \
							 resources.adio.dac.d;
  
  /* rounding */
  if (double_vlt >= 0)
    int_val = (int)(double_vlt + 0.5);
  else
    int_val = (int)(double_vlt - 0.5);

  /* if values overcome the hard limits set the limits */
  if (int_val > ADIO_HIGHTHR)
    short_val = ADIO_HIGHTHR;
  else if (int_val < -ADIO_LOWTHR)
    short_val = -ADIO_LOWTHR;
  else
    short_val = (short)int_val;

  end_vlt = short_val;

#ifdef MYDEBUG
  fprintf(stderr, "linrmp_wr(): end_vlt = 0x%x\n", end_vlt);
#endif
  
  /* read the voltage DAC setting value */
  if (equip(resources.adio.devname, ADIO_RDDAC, (char *)&short_val) != 0) {
    set_err(call_pnt, errno, ETERROR + ETAPPL, \
            "Open/Ioctl Error in Devsrv:", resources.adio.devname, ADIO_RDDAC);
    return;
  }

  start_vlt = short_val;
	
#ifdef MYDEBUG
  fprintf(stderr, "linrmp_wr(): start_vlt = 0x%x\n", start_vlt);
#endif
    
  /* calculate voltage, calibrated */
  double_vlt = start_vlt;
  double_vlt = double_vlt * double_vlt * double_vlt * resources.adio.dac_set.a + \
							 double_vlt * double_vlt * resources.adio.dac_set.b + \
               double_vlt * resources.adio.dac_set.c + \
							 resources.adio.dac_set.d;  
  
	/* if values overcome the limits set the limits */
  if (double_vlt > resources.adio.dac_set.max)
    double_vlt = resources.adio.dac_set.max;

  if (double_vlt < resources.adio.dac_set.min)
    double_vlt = resources.adio.dac_set.min;

  /* calculate the time to finish the ramp */
  time_to_finish = fabs(double1 - double_vlt) / double_vel;
	
#ifdef MYDEBUG
  fprintf(stderr, "linrmp_wr(): time_to_finish = %f\n", time_to_finish);
  fflush(stderr);
#endif
  
  /* return the time to finish */
  *((double *)call_pnt->data) = time_to_finish;
  call_pnt->bytecount = sizeof(time_to_finish);

  /* get the initial time */
  if ((start_time = get_ticks()) < 0) {
    set_err(call_pnt, errno, ETERROR+ETAPPL, \
            "get_ticks() error:", resources.adio.devname, 0);
    return;
  }
            
  /* calculate the final time */
  end_time = start_time + (int)(time_to_finish * TICKS_SEC);

  /* start the loop */
  start_loop(VLT_RAMP);
	
}  /* end linrmp_wrrr() */


/*
 * fstrmp_wp()
 * Start fast ramping
 */
void fstrmp_wp(void)
{
	double double_vlt,double1;
	int int_val;
	short short_val;

	if (check_enable() < 0)
		return;
		
  /* another ramp already running ? */
  if ((loop_var[VLT_RAMP].loop) || (loop_var[FAST_RAMP].loop))  {
    set_err(call_pnt, ER_POINTBUSY, ETERROR+ETAPPL, \
      	    "Ramp Already Running ", "", 0 );
    return;
  }
	
	/* read the voltage DAC setting value to calc "time to finish" and start value*/
  if (equip(resources.adio.devname, ADIO_RDDAC, (char *)&short_val) != 0) {
    set_err(call_pnt, errno, ETERROR + ETAPPL, \
            "Open/Ioctl Error in Devsrv:", resources.adio.devname, ADIO_RDDAC);
    return;
  }

	double1 = short_val;
  double1 = double1 * double1 * double1 * resources.adio.dac_set.a + \
						double1 * double1 * resources.adio.dac_set.b + \
            double1 * resources.adio.dac_set.c + \
						resources.adio.dac_set.d;	
	/* if values overcome the limits set the limits */
  if (double1 > resources.adio.dac_set.max) {
		set_err(call_pnt, errno, ER_RAMPABORT, \
      "Cannot start fastramp, final value exceeded", "", 0);
    return;
  }	
	if (double1 < resources.adio.dac_set.min)
    double1 = resources.adio.dac_set.min;
	
	start_time_fast = get_ticks();
	delta = pow(double1/resources.adio.max_fast_ramp,2) * (double) resources.adio.fast_time*100;
	end_time_fast = start_time_fast + resources.adio.fast_time*100 - delta;
	
	/* start the loop */
  start_loop(FAST_RAMP);
	
}  /* end vltrmp_wrrr() */


/*
 * enable_wb()
 * Enable power
 */
void enable_wb(void)
{
  char *loc_val;
  short zero = 0;

#ifdef DEBUG
  fprintf(stderr, "enable_wb(): data = %d, bytecount = %d\n", \
          call_pnt->data, call_pnt->bytecount);
#endif
  
  /* 0 or 1 to set */
  loc_val = call_pnt->data;

  if ((*loc_val != 0) && (*loc_val != 1)) {
    set_err(call_pnt, ER_OUTOFRANGE, ETERROR + ETAPPL, \
      	    "Value Must Be 0 or 1, Not", "", *loc_val);
    return;
  }	

  /* depending on the value, call the ioctl with the proper number */ 
  if (*loc_val) {  /* on */
	/* stop ramping */
	if (loop_var[VLT_RAMP].loop)
		stop_loop(VLT_RAMP);
	if (loop_var[FAST_RAMP].loop)
		stop_loop(FAST_RAMP);
		/* always set DAC to 0x0 */
  	if (equip(resources.adio.devname, ADIO_WRDAC, (char *)&zero) != 0) {
    	set_err(call_pnt, errno, ETERROR + ETAPPL, \
   	      "Open/Ioctl Error in Devsrv:", resources.adio.devname, ADIO_WRDAC);
    	return;
		}
    /* call the physical driver with the mask */
    if (equip(resources.adio.devname, ADIO_WRDIGOLON, \
              (char *)&resources.adio.enable_mask)) {
      set_err(call_pnt, errno, ETERROR + ETAPPL, "Open/Ioctl Error in Devsrv:", \
              resources.adio.devname, ADIO_WRDIGOLON);
      return;
    }

  } 
	else {  /* off, set DAC to 0x0 too */
		/* stop ramping */
		if (loop_var[VLT_RAMP].loop) 
    	stop_loop(VLT_RAMP);
		if (loop_var[FAST_RAMP].loop) 
    	stop_loop(FAST_RAMP);	
    
		if (equip(resources.adio.devname, ADIO_WRDIGOLOFF, \
              (char *)&resources.adio.enable_mask)) {
      set_err(call_pnt, errno, ETERROR + ETAPPL, "Open/Ioctl Error in Devsrv:", \
              resources.adio.devname, ADIO_WRDIGOLOFF);
      return;
    }
		/* always set DAC to 0x0 */
  	if (equip(resources.adio.devname, ADIO_WRDAC, (char *)&zero) != 0) {
    	set_err(call_pnt, errno, ETERROR + ETAPPL, \
   	      "Open/Ioctl Error in Devsrv:", resources.adio.devname, ADIO_WRDAC);
    	return;
  	}
		
  }
	
  call_pnt->bytecount = 0;
	
}  /* end enable_wb() */
   

/*
 * stat_rf()
 * Read status
 * bit 0: 0=disable - 1=enable
 * bit 1: 0=not ramping - 1= fast ramping
 */
void stat_rf(void)
{
  unsigned short s_data = 0;
  unsigned int i_data = 0;

  /* read the status from ADIO */
  if (equip(resources.adio.devname, ADIO_DIGIN , (char *)&s_data) != 0) {
    set_err(call_pnt, errno, ETERROR + ETAPPL, \
            "Open/Ioctl Error in Devsrv:", resources.adio.devname, ADIO_DIGIN);
		return;
	}
	
	if ((~s_data) & resources.adio.in_mask)
  	i_data = 1;
 
  /* add the Ramp status bit */
  if (loop_var[FAST_RAMP].loop)  
      i_data |= RAMP_MSK;
			
  *((unsigned int*)call_pnt->data) = i_data;
  call_pnt->bytecount = sizeof(i_data);

#ifdef DEBUG
	fprintf(stderr,"stat_rf: 0x%x, bytecount = %d\n", \
		i_data,call_pnt->bytecount);
#endif
	
}  /* end stat_rf() */


/*
 * go_on_loop0()
 * VLT RAMP loop 
 */
void go_on_loop0(void)
{
  int time;
  short vlt_val;

  switch (loop_var[VLT_RAMP].step) {
    case 0:		
      /* get current ticks; if fails stop ramp */
      if ((time = get_ticks()) < 0) {
        set_msg("get_ticks() error in lin ramp", "", errno);
        stop_loop(VLT_RAMP);
        return;
      }

      /* if ramp already finished set the last value and stop the loop */
      if (time >= end_time) {
        if (equip(resources.adio.devname, ADIO_WRDAC, (char *)&end_vlt) != 0)
          set_msg("EQUIP Error in Ramp", "", errno);
        stop_loop(VLT_RAMP);
				return;
      }

      /* calculate the value to set */
      vlt_val = (double)start_vlt + (double)(end_vlt - start_vlt) * \
                 (double)(time - start_time) / (double)(end_time - start_time);

      /* set the value */
      if (equip(resources.adio.devname, ADIO_WRDAC, (char *)&vlt_val) != 0) {
        set_msg("EQUIP Error in Ramp","", errno);
        stop_loop(VLT_RAMP);
        return;
      }

      /* wait and continue */
      WAIT_POINT(VLT_RAMP, RAMP_TICKS, 0)
    default:
      LOOP_DEFAULT(VLT_RAMP)
  }  /* end switch */
	
}  /* end go_on_loop0() */


/*
 * go_on_loop1()
 * FAST RAMP loop 
 */
void go_on_loop1(void)
{
  int time,int_val;
	short vlt_val;
	double double_vlt;
	
  switch (loop_var[FAST_RAMP].step) {
    case 0:
      /* get current ticks; if fails stop ramp */
      if ((time = get_ticks()) < 0) {
        set_msg("get_ticks() error in fast ramp", "", errno);
        stop_loop(FAST_RAMP);
        return;
      }

      /* if ramp already finished set the last value and stop the loop */
      if (time >= end_time_fast) {
        if (equip(resources.adio.devname, ADIO_WRDAC, (char *)&end_vlt_fast) != 0)
          set_msg("EQUIP Error in Ramp", "", errno);
        stop_loop(FAST_RAMP);
				return;
      }
									
			double_vlt = (double) resources.adio.max_fast_ramp * \
				pow((double)(delta + time - start_time_fast) / (double)resources.adio.fast_time / 100 ,0.5);			       
			/* calculate the value to set on the DAC, calibrated */
  		double_vlt = double_vlt * double_vlt * double_vlt * resources.adio.dac.a + \
							 double_vlt * double_vlt * resources.adio.dac.b + \
               double_vlt * resources.adio.dac.c + \
							 resources.adio.dac.d;
  
  		/* rounding */
  		if (double_vlt >= 0)
    		int_val = (int)(double_vlt + 0.5);
  		else
    		int_val = (int)(double_vlt - 0.5);
			/* if values overcome the hard limits set the limits */
  		if (int_val > ADIO_HIGHTHR)
    		vlt_val = ADIO_HIGHTHR;
  		else if (int_val < -ADIO_LOWTHR)
    		vlt_val = -ADIO_LOWTHR;
  		else
    		vlt_val = (short)int_val;
			
			/* set the value */
      if (equip(resources.adio.devname, ADIO_WRDAC, (char *)&vlt_val) != 0) {
        set_msg("EQUIP Error in Ramp","", errno);
        stop_loop(FAST_RAMP);
        return;
      }

      /* wait and continue */
      WAIT_POINT(FAST_RAMP, RAMP_TICKS, 0)
    default:
      LOOP_DEFAULT(VLT_RAMP)
  }  /* end switch */
	
}  /* end go_on_loop1() */


/*
 * go_on_loop()
 * discriminate loop and call proper function
 */
void go_on_loop(int loop_num)
{
  switch (loop_num) {
    case VLT_RAMP:
      go_on_loop0();
      break;
    case FAST_RAMP:
      go_on_loop1();
      break;			
    default:
      printf("%s: loop %d not implemented!", gettime(), loop_num);
  }  /* end switch */
}  /* end go_on_loop() */


/*
 * main()
 */
int main(int argc, char *argv[])
{
  /* Check the input parameters */
  if (argc < 3) {
    printf("Usage: %s <channel number> <resource file>\n", argv[0]);
    exit(0);
  }

  /* generic part initialization */
  iniz(argv[1], N_LOOPS, amtab, N_AM);

  /* read resources from resource file */
  if (read_resources(argv[2]) < 0) {
    printf("%s: %s: error reading resource file: %s\n", \
           gettime(), argv[0], res_err);
    free_res();
    die(0);
  }

  /* HW initialization */
  iniz_hw();
	iniz_sw();
	
  server_loop();  /* server main loop */

}  /* end main() */

/* EOF */
